#!/usr/bin/perl

use Test::More tests => 4;

BEGIN { use_ok('CGI')               }
BEGIN { use_ok('DBI')               }
BEGIN { use_ok('DBIx::Browse')      }
BEGIN { use_ok('DBIx::Browse::CGI') }



